@@ -1,34 +1,42 @@
 package Git::CPAN::Patch::Import;
-BEGIN {
-  $Git::CPAN::Patch::Import::VERSION = '0.4.5';
-}
+our $AUTHORITY = 'cpan:YANICK';
+$Git::CPAN::Patch::Import::VERSION = '2.4.0';
+use 5.10.0;
 
 use strict;
 use warnings;
 
+{
+    no warnings;
 use 5.010;
 
+use File::chmod ();  # must be before 'autodie' to hush the warnings
+
 use autodie;
 
 use Archive::Extract;
 $Archive::Extract::PREFER_BIN = 1;
 
-use File::chmod;
 use File::Find;
 use File::Basename;
 use File::Spec::Functions;
 use File::Temp qw(tempdir);
 use File::Path;
 use File::chdir;
+use Path::Class qw/ file /;
 use Cwd qw/ getcwd /;
 use version;
-use Git;
+use Git::Repository;
 use CLASS;
+use DateTime;
 
 use CPANPLUS;
 use BackPAN::Index;
 
+}
+
 our $BackPAN_URL = "http://backpan.perl.org/";
+our $PERL_GIT_URL = 'git://perl5.git.perl.org/perl.git';
 
 sub backpan_index {
     state $backpan = do {
@@ -47,9 +55,9 @@ sub cpanplus {
 sub _fix_permissions {
     my $dir = shift;
 
-    chmod "u+rx", $dir;
+    File::chmod::chmod "u+rx", $dir;
     find(sub {
-        -d $_ ? chmod "u+rx", $_ : chmod "u+r", $_;
+        -d $_ ? File::chmod::chmod "u+rx", $_ : File::chmod::chmod "u+r", $_;
     }, $dir);
 }
 
@@ -82,7 +90,7 @@ sub init_repo {
             }
         }
         else {
-            Git::command_noisy('init');
+            Git::Repository->run('init');
         }
     }
 
@@ -91,30 +99,24 @@ sub init_repo {
 
 
 sub releases_in_git {
-    my $repo = Git->repository;
+    my $repo = Git::Repository->new;
     return unless contains_git_revisions();
     my @releases = map  { m{\bgit-cpan-version:\s*(\S+)}x; $1 }
                    grep /^\s*git-cpan-version:/,
-                     $repo->command(log => '--pretty=format:%b');
+                     $repo->run(log => '--pretty=format:%b');
     return @releases;
 }
 
 
 sub rev_exists {
     my $rev = shift;
-    my $repo = Git->repository;
+    my $repo = Git::Repository->new;
 
-    return eval {
-        git_cmd_try {
-            $repo->command(["rev-parse", $rev], {STDERR=>1});
-        } "fail"
-    };
+    return eval { $repo->run( 'rev-parse', $rev ); };
 }
 
 
 sub contains_git_revisions {
-    my $repo = Git->repository;
-
     return unless -d ".git";
     return rev_exists("HEAD");
 }
@@ -128,19 +130,13 @@ sub import_one_backpan_release {
     # allow multiple backpan URLs to be supplied
     $backpan_urls = [ $backpan_urls ] unless (ref($backpan_urls) eq 'ARRAY');
 
-    # on windows, some Git.pm have been reported to
-    # be command_bidi_pipe-less 
-    # rt46715
-    die "your Git.pm doesn't have a command_bidi_pipe()"
-        unless defined &Git::command_bidi_pipe;
-
-    my $repo = Git->repository;
+    my $repo = Git::Repository->new;
 
     my( $last_commit, $last_version );
 
     # figure out if there is already an imported module
-    if ( $last_commit = eval { $repo->command_oneline("rev-parse", "-q", "--verify", "cpan/master") } ) {
-        $last_version = $repo->command_oneline("cpan-last-version");
+    if ( $last_commit = eval { $repo->run("rev-parse", "-q", "--verify", "cpan/master") } ) {
+        $last_version = $repo->run("cpan-last-version");
     }
 
     my $tmp_dir = File::Temp->newdir(
@@ -194,10 +190,10 @@ sub import_one_backpan_release {
 
         local $CWD = $dir;
 
-        my $write_tree_repo = Git->repository;
+        my $write_tree_repo = Git::Repository->new( work_tree => $dir ) ;
 
-        $write_tree_repo->command_noisy( qw(add -v --force .) );
-        $write_tree_repo->command_oneline( "write-tree" );
+        $write_tree_repo->run( qw(add -v --force .) );
+        $write_tree_repo->run( "write-tree" );
     };
 
     # Create a commit for the imported tree object and write it into
@@ -212,17 +208,11 @@ sub import_one_backpan_release {
     my @parents = grep { $_ } $last_commit;
 
 
-    # FIXME $repo->command_bidi_pipe is broken
-    my ( $pid, $in, $out, $ctx ) = Git::command_bidi_pipe(
-        "commit-tree", $tree,
-        map { ( -p => $_ ) } @parents,
-    );
-
     # commit message
     my $name    = $release->dist;
     my $version = $release->version || '';
-    $out->print( join ' ', ( $last_version ? "import" : "initial import of" ), "$name $version from CPAN\n" );
-    $out->print( <<"END" );
+    my $message = join ' ', ( $last_version ? "import" : "initial import of"), "$name $version from CPAN\n";
+    $message .= <<"END";
 
 git-cpan-module:   $name
 git-cpan-version:  $version
@@ -231,28 +221,21 @@ git-cpan-file:     @{[ $release->prefix ]}
 
 END
 
-    # we need to send an EOF to git in order for it to actually finalize the commit
-    # this kludge makes command_close_bidi_pipe not barf
-    close $out;
-    open $out, '<', \my $buf;
-
-    chomp(my $commit = <$in>);
-
-    Git::command_close_bidi_pipe($pid, $in, $out, $ctx);
-
+    my $commit = $repo->run( { input => $message }, 'commit-tree', $tree,
+           map { ( -p => $_ ) } @parents );
 
     # finally, update the fake branch and create a tag for convenience
     my $dist = $release->dist;
-    $repo->command_noisy('update-ref', '-m' => "import $dist", 'refs/heads/cpan/master', $commit );
+    print $repo->run('update-ref', '-m' => "import $dist", 'refs/heads/cpan/master', $commit );
 
     if( $version ) {
         my $tag = $version;
         $tag =~ s{^\.}{0.};  # git does not like a leading . as a tag name
         $tag =~ s{\.$}{};    # nor a trailing one
-        if( $repo->command( "tag", "-l" => $tag ) ) {
+        if( $repo->run( "tag", "-l" => $tag ) ) {
             say "Tag $tag already exists, overwriting";
         }
-        $repo->command_noisy( "tag", "-f" => $tag, $commit );
+        print $repo->run( "tag", "-f" => $tag, $commit );
         say "created tag '$tag' ($commit)";
     }
 }
@@ -321,13 +304,13 @@ sub import_from_backpan {
         }
     }
 
-    my $repo = Git->repository;
+    my $repo = Git::Repository->new;
     if( !rev_exists("master") ) {
-        $repo->command_noisy('checkout', '-t', '-b', 'master', 'cpan/master');
+        print $repo->run('checkout', '-t', '-b', 'master', 'cpan/master');
     }
     else {
-        $repo->command_noisy('checkout', 'master', '.');
-        $repo->command_noisy('merge', 'cpan/master');
+        print $repo->run('checkout', 'master', '.'),
+        $repo->run('merge', 'cpan/master');
     }
 
     return $repo_dir;
@@ -335,89 +318,108 @@ sub import_from_backpan {
 
 
 sub fixup_repository {
-    my $repo = Git->repository;
+    my $repo = Git::Repository->new;
 
     return unless -d ".git";
 
     # We do our work in cpan/master, it might not exist if this
     # repo was cloned from gitpan.
     if( !rev_exists("cpan/master") and rev_exists("master") ) {
-        $repo->command_noisy('branch', '-t', 'cpan/master', 'master');
+        print $repo->run('branch', '-t', 'cpan/master', 'master');
     }
 }
 
+use MetaCPAN::API;
+my $mcpan = MetaCPAN::API->new;
+
+sub find_release {
+    my $input = shift;
+
+    return eval { $mcpan->release(
+                        distribution => $mcpan->module($input)->{distribution}
+                  ) }
+        || eval { $mcpan->release( distribution => $input ) }
+        || die "could not find release for '$input' on metacpan\n";
+
+}
 
 sub main {
     my $module = shift;
     my $opts   = shift;
 
-    return import_from_gitpan( $module, $opts ) if $opts->{gitpan};
-
     if ( delete $opts->{backpan} ) {
         return import_from_backpan( $module, $opts );
     }
 
-    my $full_hist;
-
-    my $repo = Git->repository;
+    my $repo = Git::Repository->new;
 
     my ( $last_commit, $last_version );
 
     # figure out if there is already an imported module
-    if ( $last_commit = eval { $repo->command_oneline("rev-parse", "-q", "--verify", "cpan/master") } ) {
-        $module     ||= $repo->command_oneline("cpan-which");
-        $last_version = $repo->command_oneline("cpan-last-version");
+    if ( $last_commit = eval { $repo->run("rev-parse", "-q", "--verify", "cpan/master") } ) {
+        $module     ||= $repo->run("cpan-which");
+        $last_version = $repo->run("cpan-last-version");
     }
 
-    die("Usage: git cpan-import Foo::Bar\n") unless $module;
+    die("Usage: git-cpan import Foo::Bar\n") unless $module;
 
     # first we figure out a module object from the module argument
-    # CPANPLUS handles dist names and URIs too
+
+    my $release = find_release($module);
 
     # based on the version number it figured out for us we decide whether or not to
     # actually import.
 
-    my $cpan = CPANPLUS::Backend->new;
-    my $module_obj = $cpan->parse_module( module => $module ) or die("No such module $module");
+    my $name    = $release->{name};
+    my $version = $release->{version};
+    my $dist    = $release->{distribution};
 
-    my $name    = $module_obj->name;
-    my $version = $module_obj->version;
-    my $dist    = $module_obj->package;
-    my $dist_name = join("-", $module_obj->package_name, $module_obj->package_version);
+     if ( $dist eq 'perl' ) {
+        say "$name is a core modules, ",
+            "clone perl from $PERL_GIT_URL instead.";
+        exit;
+    }
 
-    my $prettyname = $name . ( " ($module)" x ( $name ne $module ) );
+    my $prettyname = $dist . ( " ($module)" x ( $dist ne $module ) );
 
     if ( $last_version and $opts->{checkversion} ) {
         # if last_version is defined this is an update
         my $imported = version->new($last_version);
-        my $will_import = version->new($module_obj->version);
+        my $will_import = version->new($release->{version});
+
+        die "$name has already been imported\n" if $imported == $will_import;
 
-        die "$dist_name has already been imported\n" if $imported == $will_import;
-    
         die "imported version $imported is more recent than $will_import, can't import\n"
           if $imported > $will_import;
 
         say "updating $prettyname from $imported to $will_import";
-    
+
     } else {
         say "importing $prettyname";
     }
 
+    require LWP::UserAgent;
 
+    my $ua = LWP::UserAgent->new;
 
     # download the dist and extract into a temporary directory
-
-    my $tmp_dir = tempdir( CLEANUP => 1 );
+    my $tmp_dir = tempdir( CLEANUP => 0 );
 
     say "downloading $dist";
 
-    my $location = $module_obj->fetch( fetchdir => $tmp_dir )
-      or die "couldn't retrieve distribution file for module $module";
+    my $tarball = file( $tmp_dir, $release->{archive} );
+
+    $ua->mirror(
+        $release->{download_url} => $tarball
+    ) or die "couldn't fetch tarball\n";
 
     say "extracting distribution";
 
-    my $dir = $module_obj->extract( extractdir => $tmp_dir )
-      or die "couldn't extract distribution file $location";
+    my $archive = Archive::Extract->new( archive => $tarball );
+    $archive->extract( to => $tmp_dir );
+
+    my $dist_dir = $archive->extract_path
+        or die "extraction failed\n";
 
     # create a tree object for the CPAN module
     # this imports the source code without touching the user's working directory or
@@ -427,41 +429,39 @@ sub main {
         # don't overwrite the user's index
         local $ENV{GIT_INDEX_FILE} = catfile($tmp_dir, "temp_git_index");
         local $ENV{GIT_DIR} = catfile( getcwd(), '.git' );
-        local $ENV{GIT_WORK_TREE} = $dir;
+        local $ENV{GIT_WORK_TREE} = $dist_dir;
 
-        local $CWD = $dir;
+        local $CWD = $dist_dir;
 
-        my $write_tree_repo = Git->repository;
+        my $write_tree_repo = Git::Repository->new( work_tree => $dist_dir );
 
-        $write_tree_repo->command_noisy( qw(add -v --force .) );
-        $write_tree_repo->command_oneline( "write-tree" );
+        $write_tree_repo->run( qw(add -v --force .) );
+        $write_tree_repo->run( "write-tree" );
     };
 
-
-
-
-
-    # reate a commit for the imported tree object and write it into
+    # create a commit for the imported tree object and write it into
     # refs/heads/cpan/master
 
     {
         local %ENV = %ENV;
 
-        my $author_obj = $module_obj->author;
+        my $author_obj = $mcpan->author($release->{author});
 
         # try to find a date for the version using the backpan index
         # secondly, if the CPANPLUS author object is a fake one (e.g. when importing a
         # URI), get the user object by using the ID from the backpan index
         unless ( $ENV{GIT_AUTHOR_DATE} ) {
             my $mtime = eval {
-                return if $author_obj->isa("CPANPLUS::Module::Author::Fake");
-                my $checksums = $module_obj->checksums;
-                my $href = $module_obj->_parse_checksums_file( file => $checksums );
-                return $href->{$dist}{mtime};
+                DateTime->from_epoch( epoch => $release->{stat}{mtime})->ymd;
             };
 
             warn $@ if $@;
 
+            # CPAN::Checksums makes YYYY-MM-DD dates, but GIT_AUTHOR_DATE
+            # doesn't support that. 
+            $mtime .= 'T00:00::00'
+                if $mtime =~ m/\A (\d\d\d\d) - (\d\d?) - (\d\d?) \z/x;
+
             if ( $mtime ) {
                 $ENV{GIT_AUTHOR_DATE} = $mtime;
             } else {
@@ -471,7 +471,8 @@ sub main {
                     # we need the backpan index for dates
                     my $backpan = $CLASS->backpan_index;
 
-                    %dists = map { $_->filename => $_ } $backpan->releases($module_obj->package_name);
+                    %dists = map { $_->filename => $_ }
+                    $backpan->releases($release->{name});
                 }
 
                 if ( my $bp_dist = $dists{$dist} ) {
@@ -479,7 +480,7 @@ sub main {
                     $ENV{GIT_AUTHOR_DATE} = $bp_dist->date;
 
                     if ( $author_obj->isa("CPANPLUS::Module::Author::Fake") ) {
-                        $author_obj = $cpan->author_tree($bp_dist->cpanid);
+                        $author_obj = $mcpan->author_tree($bp_dist->cpanid);
                     }
                 } else {
                     say "Couldn't find upload date for $dist";
@@ -492,96 +493,74 @@ sub main {
         }
 
         # create the commit object
-        $ENV{GIT_AUTHOR_NAME}  = $author_obj->author unless $ENV{GIT_AUTHOR_NAME};
-        $ENV{GIT_AUTHOR_EMAIL} = $author_obj->email unless $ENV{GIT_AUTHOR_EMAIL};
+        $ENV{GIT_AUTHOR_NAME}  = $author_obj->{name} unless $ENV{GIT_AUTHOR_NAME};
+        $ENV{GIT_AUTHOR_EMAIL} = $author_obj->{email}[0] unless $ENV{GIT_AUTHOR_EMAIL};
 
         my @parents = ( grep { $_ } $last_commit, @{ $opts->{parent} || [] } );
 
-        # FIXME $repo->command_bidi_pipe is broken
-        my ( $pid, $in, $out, $ctx ) = Git::command_bidi_pipe(
-            "commit-tree", $tree,
-            map { ( -p => $_ ) } @parents,
-        );
-
-        # commit message
-        $out->print( join ' ', ( $last_version ? "import" : "initial import of" ), "$name $version from CPAN\n" );
-        $out->print( <<"END" );
+        my $message = join ' ',
+            ( $last_version ? "import" : "initial import of" ),
+            "$name $version from CPAN\n";
+        $message .= <<"END";
 
 git-cpan-module:   $name
 git-cpan-version:  $version
-git-cpan-authorid: @{[ $author_obj->cpanid ]}
+git-cpan-authorid: @{[ $author_obj->{pauseid} ]}
 
 END
 
-
-        # we need to send an EOF to git in order for it to actually finalize the commit
-        # this kludge makes command_close_bidi_pipe not barf
-        close $out;
-        open $out, '<', \my $buf;
-
-        chomp(my $commit = <$in>);
-
-        Git::command_close_bidi_pipe($pid, $in, $out, $ctx);
-
+        my $commit = $repo->run(
+            { input => $message },
+            'commit-tree', $tree, map { ( -p => $_ ) } @parents );
 
         # finally, update the fake remote branch and create a tag for convenience
 
-        $repo->command_noisy('update-ref', '-m' => "import $dist", 'refs/remotes/cpan/master', $commit );
+        print $repo->run('update-ref', '-m' => "import $dist", 'refs/remotes/cpan/master', $commit );
 
-        $repo->command_noisy( tag => $version, $commit );
+        print $repo->run( tag => $version, $commit );
 
         say "created tag '$version' ($commit)";
     }
-}
-
-sub import_from_gitpan {
-    my ( $module, $opts ) = @_;
-
-    die "Usage: git cpan-import --gitpan Foo::Bar\n" unless $module;
-
-    my $cpan = CPANPLUS::Backend->new;
-    my $module_obj = $cpan->parse_module( module => $module )
-      or die "no such module $module\n";
-
-    my $dist = $module_obj->name;
-    $dist =~ s/::/-/g;
-
-    my $repo = Git->repository;
-    my $url  = "git://github.com/gitpan/${dist}.git";
-
-    unless ( "gitpan\n" ~~ $repo->command('remote') ) {
-
-        # no gitpan remote? create it!
-
-        # but first, does it exist?
-
-        die "no repo found at $url\n"
-          unless eval { $repo->command( 'ls-remote', $url ) };
-
-        $repo->command_noisy( 'remote', 'add', 'gitpan', $url );
-    }
-
-    $repo->command_noisy(qw/ fetch gitpan /);
 
 }
 
 1;
 
-__END__
+=pod
+
+=encoding UTF-8
 
 =head1 NAME
 
-Git::CPAN::Patch::Import - The meat of git-cpan-import
+Git::CPAN::Patch::Import
 
 =head1 VERSION
 
-version 0.4.5
+version 2.4.0
 
 =head1 DESCRIPTION
 
 This is the guts of Git::CPAN::Patch::Import moved here to make it callable
 as a function so git-backpan-init goes faster.
 
+=head1 NAME
+
+Git::CPAN::Patch::Import - The meat of git-cpan-import
+
+=head1 AUTHOR
+
+Yanick Champoux <yanick@cpan.org>
+
+=head1 COPYRIGHT AND LICENSE
+
+This software is copyright (c) 2021, 2018, 2017, 2016, 2015, 2014, 2013, 2012, 2011, 2010, 2009 by Yanick Champoux.
+
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
+
 =cut
 
-1;
\ No newline at end of file
+__END__
+
+
+1;
